# High Performance Distributed Object Store [HPDOS]
## About
The HPDOS project is an attempt to optimise the storage stack in hyperconverged setups in recognition of recent advances in hardware technologies like SmartNICs, NVMe SSDs and persistent memories. 

Here we plan to take advantage of SmartNIC technologies more specifically SoC based NICs to speed up the storage stack.

## Directory
```
├── code
│   ├── hpdos_client
│   ├── hpdos_server
│   └── README.md
├── code_p4_tryouts
│   ├── client
│   ├── controller
│   ├── generator
│   ├── INSTRUCTIONS.txt
│   ├── LICENSE
│   ├── LiteratureSurvey.md
│   ├── MeetingMinutes.md
│   ├── mininet
│   ├── nc_config.py
│   ├── p4src
│   ├── README.md
│   └── server
├── documentation
│   ├── designs
│   ├── dfs_report.pdf
│   ├── handover-material-rinku
│   ├── HPDOS-presentation.pdf
│   └── references
├── mtcp-dpdk
│   ├── dpdk-19.11.8.tar.xz
│   ├── mtcp-devel.tar.xz
│   └── README
├── RAMCloud Offload
│   ├── client.cpp
│   ├── master.cpp
│   ├── no_offload_master.cpp
│   └── offload.cpp
└── README.md
```

### HPDOS NO_OFFLOAD/ FULL_OFFLOAD implementation

For details on the kernel stack based implementation on the NO_OFFLOAD or FULL_OFFLOAD HPDOS stack look into the [code](./code) folder
- For implementations on client cache design use the client_cache branch
- For implementations using LSM tree metadata_lsmtree branch

### MISC

- [code_p4_tryouts](./code_p4_tryouts) contain a bunch of early experimental designs in __P4__ on __caching__ for distributed KV stores based on Netcache design.
- For our efforts on __RAMCloud offloading__ checkout shashank-* branches
- For __documenation__, references and bunch of early phase designs look into [documentation](./documentation) directory.
- [mtcp-dpdk](./mtcp-dpdk) contains Bluefield compatible binaries and installation instructions for __mTCP and DPDK__.

#### Setup
- For setting up the Bluefield NIC in __embedded mode__ read the [SmartNIC setup readme](./documentation/smartnic_setup.md).
- For setup related to __Mediated Devices__ in embedded mode read the [Mediated Devices readme](./documentation/mediated_devices.md).
- For setting up __separated mode__ or information regarding on __DPDK mTCP__ checkout [handover-material-rinku](./documentation/handover-material-rinku).